/* BackStep.mq4

  Description:
    1. Place an order, setting TakeProfit level.
    2. If market reverses direction, place subsequent orders at Pips intervals, doubling the lot size each time.
    3. When the last order reaches its TakeProfit level, close all orders.

  Ref: 10points_3.mq4  Alejandro Galindo  elCactus.com

  Revision History:
  11/06/08  v0.1  Decompiled.
  11/10/08  v0.2  Rewritten. Fixed StopLoss calculation. Fixed net profit calculation. Added blackout period.
  11/10/08  v0.3  Removed max lot=100 restriction.
  11/11/08  v0.4  Fixed timezone bug. Fixed trailing stop bug. Fixed net profit calculation.
                  Close orders all at once instead of one/tick. Adjust all trailing stops at once.
  11/15/08  v0.5  Modified for mini/micro accts. Pips are now doubles for fractional pricing. Improved screen comments.
                  Removed TrailingStop,OrdersToProtect,SecureProfit.
  12/08/08  v0.6  Fixed problem with Interbank FX mini account (symbol name had "m" appended).
*/
#property copyright "v0.6 Copyright  2008, Kevin Spreier"
#property link      "kevin@spreier.com"

#include <stderror.mqh>  // error handling
#include <stdlib.mqh>    // error handling

// set Allow DLL Imports and unset Confirm DLL Function Calls
#import "kernel32.dll"
int GetTimeZoneInformation(int& TZInfoArray[]);
#import
#define TIME_ZONE_ID_UNKNOWN  0
#define TIME_ZONE_ID_STANDARD 1
#define TIME_ZONE_ID_DAYLIGHT 2
int      NewYorkTZ;  // time zone offset in minutes ET - local
datetime NewYork;    // current time in New York
int      BeginHour;  // opening bell
int      EndHour;    // closing bell

extern double   TakeProfit = 10.0;         // take profit pips; shouldn't be > 2*Pips
extern double   StopLoss = 0.0;            // stop loss pips
extern double   Pips = 5.0;                // distance in pips from one order to another
extern bool     MoneyManagement = FALSE;   // T=set lot size based on account size
extern double   Lots = 0.01;               // lot size
extern bool     AccountProtection = FALSE; // T=on F=off
extern int      MaxTrades = 5;             // max orders to open
extern bool     ReverseCondition = FALSE;  // T=reverse decision to go long/short
extern bool     BlackoutPeriod = FALSE;    // limit trading hours
extern datetime BeginTrading = D'2008.11.01 18:00:00';  // time of day (Eastern Time) to begin trading
extern datetime EndTrading   = D'2008.11.02 02:00:00';  // time of day (Eastern Time) to end trading

// account size - a pip is 0.0001. At 100/1 leverage:
#define STANDARD 0  // 1 full lot costs $1,000 and controls $100,000 currency; each pip is worth $10.00
#define MINI     1  // 1 mini lot costs $100 and controls $10,000 currency; each pip is worth $1.00
#define MICRO    2  // 1 micro lot costs $10 and controls $1,000 currency; each pip is worth $0.10
int AccountSize;

// TODO: add magic #
int    PreviousOpenOrders = 0;
int    LotDigits;
double Slippage;

int init()
{
  // calc account size
  AccountSize = MarketInfo("EURGBP",MODE_DIGITS) - 4;  // digits=4 for standard size EURGBP account
  if (AccountSize==-4)  AccountSize=MarketInfo("EURGBPm",MODE_DIGITS)-4;  // not found; try mini symbol
  switch (AccountSize)
  {
    case STANDARD:
      string acct = "Standard";
      break;
    case MINI:
      acct = "Mini";
      break;
    case MICRO:
      acct = "Micro";
      break;
    default:
      acct = "Unknown";
      break;
  }
  // multiply all pips to compensate for extra digit(s)
  TakeProfit   *= MathPow(10.0,AccountSize);
  StopLoss     *= MathPow(10.0,AccountSize);
  Pips         *= MathPow(10.0,AccountSize);
  Slippage = Pips;  // init slippage
Print("digits="+MarketInfo("EURGBP",MODE_DIGITS)," as="+AccountSize," acct="+acct," mult="+MathPow(10.0,AccountSize));

  // calc lot digits
  LotDigits = getDigits(MarketInfo(Symbol(),MODE_LOTSTEP));  // count of digits after decimal point

  // calc time zone offset for New York
  int TZInfoArray[43];
  int tz = GetTimeZoneInformation(TZInfoArray);  // local time zone
  NewYorkTZ = 5*60 - TZInfoArray[0];
  if (tz==TIME_ZONE_ID_STANDARD)  NewYorkTZ += TZInfoArray[21];  // add standard time bias
  if (tz==TIME_ZONE_ID_DAYLIGHT)  NewYorkTZ += TZInfoArray[42];  // compensate for daylight savings

  // calc current time in New York
  NewYork   = TimeLocal() - NewYorkTZ*60;
  BeginHour = TimeHour(BeginTrading);
  EndHour   = TimeHour(EndTrading);

  Comment(TimeToStr(NewYork,TIME_DATE|TIME_SECONDS), " ET",
          "\n", AccountCompany(), " Server: ", AccountServer(),
          "\n", acct, " Account #", AccountNumber(), "  Leverage: ", AccountLeverage(), ":1  Min Lot: ", MarketInfo(Symbol(),MODE_MINLOT), "  Max Lot: ", MarketInfo(Symbol(),MODE_MAXLOT),
          "\n", "Account Balance:  $", DoubleToStr(AccountBalance(),2));
}

int deinit()
{
  Comment("");
}

int start()
{
  int    OpenOrders = 0;
  int    LastOrderType = -1;
  double LastPrice = 0.0;
  for (int i=0; i<OrdersTotal(); i++)
  {
    // count open orders
    if (!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))  continue;
    if (OrderSymbol() == Symbol())
    {
      OpenOrders++;
      LastOrderType = OrderType();
      LastPrice = OrderOpenPrice();
    }
  }

  if (OpenOrders>0 && PreviousOpenOrders>OpenOrders)
  {
    // close all orders
    for (i=OrdersTotal()-1; i>= 0; i--)
    {
      if (!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))  continue;
      if (OrderSymbol() == Symbol())
      {
        if (OrderType()==OP_BUY)        OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, Blue);
        else if (OrderType()==OP_SELL)  OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, Red);
        Comment("Closing all orders");
      }
    }
    return;
  }
  PreviousOpenOrders = OpenOrders;

  NewYork = TimeLocal() - NewYorkTZ * 60;
  if (BlackoutPeriod
      && !(TimeHour(NewYork)>=BeginHour && TimeHour(NewYork)<EndHour)
      && !(BeginHour>EndHour && TimeHour(NewYork)>=BeginHour)
      && !(BeginHour>EndHour && TimeHour(NewYork)<EndHour))
  {
    // blackout period
    PreviousOpenOrders++;  // force all open orders to close at closing bell
    Comment(TimeToStr(NewYork,TIME_DATE|TIME_SECONDS), " ET  <BLACKOUT>");
    return;
  }

  if (OpenOrders < 1)
  {
    // guess market direction
    if (iMACD(NULL, 0, 14, 26, 9, PRICE_CLOSE, MODE_MAIN, 1) > iMACD(NULL, 0, 14, 26, 9, PRICE_CLOSE, MODE_MAIN, 2))  LastOrderType = OP_BUY;
    if (iMACD(NULL, 0, 14, 26, 9, PRICE_CLOSE, MODE_MAIN, 1) < iMACD(NULL, 0, 14, 26, 9, PRICE_CLOSE, MODE_MAIN, 2))  LastOrderType = OP_SELL;
    if (ReverseCondition)
    {
      if (LastOrderType==OP_BUY)        LastOrderType = OP_SELL;
      else if (LastOrderType==OP_SELL)  LastOrderType = OP_BUY;
    }
  }

  string money = "";
  if (OpenOrders < MaxTrades)
  {
    bool placeOrder = FALSE;
    if (LastOrderType == OP_BUY)
    {
      // long position
      double price = Ask;
      double sl = price - StopLoss*Point;
      double tp = price + TakeProfit*Point;
      color clr = Blue;
      string str = "buy";
      placeOrder = ((LastPrice-price)>=Pips*Point);
    }
    if (LastOrderType == OP_SELL)
    {
      // short position
      price = Bid;
      sl = price + StopLoss*Point;
      tp = price - TakeProfit*Point;
      clr = Red;
      str = "sell";
      placeOrder = ((price-LastPrice)>=Pips*Point);
    }
    if (placeOrder || OpenOrders<1)
    {
      // calc lot size
      if (MoneyManagement)  double lots = AccountBalance() / MarketInfo(Symbol(),MODE_MARGINREQUIRED) / MathPow(10.0,AccountSize);
      else                  lots = Lots;  // fixed lot size
      lots = NormalizeDouble(lots,LotDigits);
      if (OpenOrders != 0)
      {
        // double the previous lot size
        for (int j=0; j<OpenOrders; j++)
        {
          if (MaxTrades>12)  lots = NormalizeDouble(1.5*lots,LotDigits);
          else               lots = NormalizeDouble(2.0*lots,LotDigits);
        }
      }
      lots = MathMax(lots,MarketInfo(Symbol(),MODE_MINLOT));
      lots = MathMin(lots,MarketInfo(Symbol(),MODE_MAXLOT));
      if (AccountFreeMarginCheck(Symbol(),LastOrderType,lots)<=0.0 || GetLastError()==ERR_NOT_ENOUGH_MONEY)
      {
//Print("lots="+lots," LOTSIZE="+MarketInfo(Symbol(),MODE_LOTSIZE)," Leverage="+AccountLeverage()," TICKSIZE="+MarketInfo(Symbol(),MODE_TICKSIZE)," EURUSD="+MarketInfo("EURUSD",MODE_ASK)," Ask="+Ask);
        money = "\nNot enough money (" + DoubleToStr(lots,LotDigits) + " lots";
        double eurusd = MarketInfo("EURUSD",MODE_ASK);  // this will be 0.0000 in testing mode
        if (eurusd != 0.0)
        {
          double tickValue = MarketInfo(Symbol(),MODE_LOTSIZE)*MarketInfo(Symbol(),MODE_TICKSIZE)*eurusd/Ask;
          money = money + "; need Free Margin: $" + DoubleToStr(lots*MarketInfo(Symbol(),MODE_LOTSIZE)/AccountLeverage()*tickValue,2);
        }
        money = money + ")";
      } else
      {
// TODO: add comment
        // open a position
        if (StopLoss==0.0)    sl = 0.0;
        if (TakeProfit==0.0)  tp = 0.0;
        int ticket = OrderSend(Symbol(),LastOrderType,lots,price,Slippage,NormalizeDouble(sl,Digits),NormalizeDouble(tp,Digits),NULL,0,0,clr);
        str = "Opening " + str + " order @" + DoubleToStr(price,Digits);
        if (ticket==-1)  str = str + " failed; " + ErrorDescription(GetLastError());
        Comment(str);
        return;
      }
    }
  }

  // display status
  if (OrderType()==OP_BUY)        {double nextPrice=LastPrice-Pips*Point; double close=(Bid-OrderTakeProfit())/Point;}
  else if (OrderType()==OP_SELL)  {       nextPrice=LastPrice+Pips*Point;        close=(OrderTakeProfit()-Ask)/Point;}
  double sol = AccountStopoutLevel();  // must convert int to double for math calculation
  string cmt = TimeToStr(NewYork,TIME_DATE|TIME_SECONDS)+" ET" + "  StopOut: "+AccountStopoutLevel()+"%"
             + "\n# Orders="+PreviousOpenOrders + "  LastPrice="+DoubleToStr(LastPrice,Digits) + "  LastLots="+DoubleToStr(OrderLots(),LotDigits) + "  Close="+DoubleToStr(close,0)
             + "\nSpread="+DoubleToStr((Ask-Bid)/MarketInfo(Symbol(),MODE_TICKSIZE),0) + "  NextPrice="+DoubleToStr(nextPrice,Digits) + "  Type=";
  switch (LastOrderType)
  {
    case OP_BUY:
      cmt = cmt + "LONG";
      break;
    case OP_SELL:
      cmt = cmt + "SHORT";
      break;
    default:
      cmt = cmt + "NONE";
      break;
  }
  cmt = cmt + money;
  if (IsTesting())
  {
    for (i=0; i<OrdersTotal(); i++)
    {
      if (!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))  continue;
      if (OrderSymbol() != Symbol())  continue;
      cmt = cmt + "\n"+OrderTicket()+" "+OrderType()+" "+DoubleToStr(OrderLots(),2)+" "+DoubleToStr(OrderOpenPrice(),Digits)+" "+DoubleToStr(OrderStopLoss(),Digits)+" "+DoubleToStr(OrderTakeProfit(),Digits)+" "+DoubleToStr(OrderClosePrice(),Digits)+" "+DoubleToStr(OrderSwap(),2)+" "+DoubleToStr(OrderProfit(),2);
    }
  }
  Comment(cmt);
}

// Returns the number of decimal places in d.
int getDigits(double d)
{
  int n = 0;
  d = MathMod(d,1.0);
  while (MathFloor(d) == 0.0)
  {
    d *= 10.0;
    n++;
  }
  return (n);
}